@@ -1,10 +1,12 @@
 const PassThrough = require('stream').PassThrough;
 const getInfo = require('./info');
-const util = require('./util');
+const utils = require('./utils');
+const formatUtils = require('./format-utils');
+const urlUtils = require('./url-utils');
 const sig = require('./sig');
 const miniget = require('miniget');
 const m3u8stream = require('m3u8stream');
-const parseTime = require('m3u8stream/dist/parse-time');
+const { parseTimestamp } = require('m3u8stream');
 
 
 /**
@@ -23,24 +25,26 @@ module.exports = ytdl;
 
 ytdl.getBasicInfo = getInfo.getBasicInfo;
 ytdl.getInfo = getInfo.getInfo;
-ytdl.chooseFormat = util.chooseFormat;
-ytdl.filterFormats = util.filterFormats;
-ytdl.validateID = util.validateID;
-ytdl.validateURL = util.validateURL;
-ytdl.getURLVideoID = util.getURLVideoID;
-ytdl.getVideoID = util.getVideoID;
+ytdl.chooseFormat = formatUtils.chooseFormat;
+ytdl.filterFormats = formatUtils.filterFormats;
+ytdl.validateID = urlUtils.validateID;
+ytdl.validateURL = urlUtils.validateURL;
+ytdl.getURLVideoID = urlUtils.getURLVideoID;
+ytdl.getVideoID = urlUtils.getVideoID;
 ytdl.cache = {
   sig: sig.cache,
   info: getInfo.cache,
+  watch: getInfo.watchPageCache,
   cookie: getInfo.cookieCache,
 };
+ytdl.version = require('../package.json').version;
 
 
 const createStream = options => {
   const stream = new PassThrough({
     highWaterMark: (options && options.highWaterMark) || 1024 * 512,
   });
-  stream.destroy = () => { stream._isDestroyed = true; };
+  stream._destroy = () => { stream.destroyed = true; };
   return stream;
 };
 
@@ -66,7 +70,7 @@ const pipeAndSetEvents = (req, stream, end) => {
 const downloadFromInfoCallback = (stream, info, options) => {
   options = options || {};
 
-  let err = util.playError(info, 'UNPLAYABLE');
+  let err = utils.playError(info.player_response, ['UNPLAYABLE', 'LIVE_STREAM_OFFLINE', 'LOGIN_REQUIRED']);
   if (err) {
     stream.emit('error', err);
     return;
@@ -79,13 +83,13 @@ const downloadFromInfoCallback = (stream, info, options) => {
 
   let format;
   try {
-    format = util.chooseFormat(info.formats, options);
+    format = formatUtils.chooseFormat(info.formats, options);
   } catch (e) {
     stream.emit('error', e);
     return;
   }
   stream.emit('info', info, format);
-  if (stream._isDestroyed) { return; }
+  if (stream.destroyed) { return; }
 
   let contentLength, downloaded = 0;
   const ondata = chunk => {
@@ -143,7 +147,7 @@ const downloadFromInfoCallback = (stream, info, options) => {
         req = miniget(format.url, requestOptions);
         req.on('data', ondata);
         req.on('end', () => {
-          if (stream._isDestroyed) { return; }
+          if (stream.destroyed) { return; }
           if (end && end !== rangeEnd) {
             start = end + 1;
             end += dlChunkSize;
@@ -156,7 +160,7 @@ const downloadFromInfoCallback = (stream, info, options) => {
     } else {
       // Audio only and video only formats don't support begin
       if (options.begin) {
-        format.url += `&begin=${parseTime.humanStr(options.begin)}`;
+        format.url += `&begin=${parseTimestamp(options.begin)}`;
       }
       if (options.range && (options.range.start || options.range.end)) {
         requestOptions.headers = Object.assign({}, requestOptions.headers, {
@@ -165,7 +169,7 @@ const downloadFromInfoCallback = (stream, info, options) => {
       }
       req = miniget(format.url, requestOptions);
       req.on('response', res => {
-        if (stream._isDestroyed) { return; }
+        if (stream.destroyed) { return; }
         contentLength = contentLength || parseInt(res.headers['content-length']);
       });
       req.on('data', ondata);
@@ -173,12 +177,10 @@ const downloadFromInfoCallback = (stream, info, options) => {
     }
   }
 
-  stream.destroy = () => {
-    stream._isDestroyed = true;
-    if (req.abort) req.abort();
+  stream._destroy = () => {
+    stream.destroyed = true;
+    req.destroy();
     req.end();
-    req.removeListener('data', ondata);
-    req.unpipe();
   };
 };
 
